;;;; -*- Mode: Lisp -*-
;;;; $Id: update-mib.lisp 862 2011-03-20 06:52:18Z binghe $

(in-package :snmp)

(defparameter *preload-mibs*
  '(#p"MIB:IETF;SNMPv2-SMI"
    #p"MIB:IETF;SNMPv2-TC"
    #p"MIB:IETF;SNMPv2-TM"
    #p"MIB:IETF;SNMPv2-CONF"
    #p"MIB:IETF;SNMPv2-MIB"
    #p"MIB:IANA;IANAifType-MIB"

    #p"MIB:LISP;LISP-MIB.TXT"
    #p"MIB:LISP;ABCL-MIB.TXT"
    #p"MIB:LISP;CLOZURE-MIB.TXT"
    #p"MIB:LISP;CMUCL-MIB.TXT"
    #p"MIB:LISP;ECL-MIB.TXT"
    #p"MIB:LISP;FRANZ-MIB.TXT"
    #p"MIB:LISP;LISPWORKS-MIB.TXT"
    #p"MIB:LISP;SBCL-MIB.TXT"
    #p"MIB:LISP;SCL-MIB.TXT"))

(defun compile-mib (&rest args)
  (apply #'compile-asn.1 args))

(defun load-mib (&rest args)
  (apply #'load-asn.1 args))

(defvar *mib-expression* #p"SNMP:MIB.LISP-EXPR")
(defvar *mib-dependency-file* #p"SNMP:MIB-DEPEND.LISP")

(defvar *pathname-base*
  (translate-logical-pathname #p"SNMP:"))

(eval-when (:compile-toplevel :load-toplevel :execute)
  (defmacro make-mib-pathname (type pathname)
    `(merge-pathnames (make-pathname :name (pathname-name (translate-logical-pathname ,pathname))
                                     :type ,type
                                     :directory (append (pathname-directory *pathname-base*)
                                                        '("compiled-mibs")))
                      *pathname-base*)))

(defun lisp-file (pathname)
  (make-mib-pathname "lisp" pathname))

(defun expr-file (pathname)
  (make-mib-pathname "lisp-expr" pathname))

(defun update-mib (&optional (mib-list *preload-mibs*))
  "Update mib.lisp-expr"
  (let ((mib.lisp-expr '())
        (mib-depend.lisp '())
        (*package* *asn.1-package*))
    (dolist (i mib-list)
      (format t "; Compiling ~A~%" i)
      (compile-asn.1 i :to (lisp-file i))
      (let ((depends (with-open-file (s (expr-file i) :direction :input)
                       (read s)))
            (name (string-downcase (pathname-name (lisp-file i)))))
        (push (if (cdr depends)
                  `(:file ,name
                    :depends-on ,(mapcar #'(lambda (x) (string-downcase (symbol-name x)))
                                         (cdr depends)))
                `(:file ,name))
              mib.lisp-expr)
        (if (cdr depends)
            (push depends mib-depend.lisp))))
    ;;; Update MIB Dependency, it's for ASDF
    (with-open-file (s *mib-expression*
                       :direction :output
                       :if-exists :supersede)
      (format s ";;;; -*- Mode: Lisp -*-~%;;;; Generated by #p\"SNMP:UPDATE-MIB.LISP\"~%")
      (pprint (setf mib.lisp-expr (nreverse mib.lisp-expr)) s)
      (terpri s))
    ;;; Update MIB Dependency, it's for MIB Browser
    (with-open-file (s *mib-dependency-file*
                       :direction :output
                       :if-exists :supersede)
      (format s ";;;; -*- Mode: Lisp -*-~%;;;; Generated by #p\"SNMP:UPDATE-MIB.LISP\"~%")
      (dolist (i `((in-package :asn.1)
                   :break-line
                   (eval-when (:load-toplevel :execute)
                     (mapcar #'(lambda (asn.1::x)
                                 (setf (gethash (car asn.1::x)
                                                asn.1:*mib-module-dependency*)
                                       (cdr asn.1::x)))
                             ',mib-depend.lisp))))
        (if (eq i :break-line)
            (terpri s)
          (pprint i s)))
      (terpri s))
    (asdf:load-system :snmp-mib)
    (pprint mib.lisp-expr)))
